###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__doc__ = """Widget tag."""
__version__ = "0.0.1"


from DocumentTemplate import DT_Util, DT_String, VSEval
import html_widgets
import composite_widgets
import types


class widget_tag:
	"""Tag for displaying widgets."""

	name='widget'
	expr = None


	def get_type(self):
		"""Returns the type of widget."""

		return self.html_widget.type


	def __init__(self, arguments):

		parameters = DT_Util.parse_params(arguments, name='', expr='', html_quote='', multiple='',
						options=[], size='', rows='', cols='', selected=[],
						rowspan='', colspan='', onclick='', onchange='', value='',
						true='', false='')

		self.type = parameters['']
		self.html_quote = parameters.has_key('html_quote')

		try:

			html_name = parameters['name']

		except KeyError:

			html_name = None

		try:

			size = parameters['size']

		except KeyError:

			size = None

		if self.type == 'text':

			self.html_widget = \
				html_widgets.text(name=html_name, size=size)
			self.value_expression = parameters['value']

		elif self.type == 'textarea':

			try:

				rows = parameters['rows']
				cols = parameters['cols']

			except KeyError:

				try:

					rows = parameters['rowspan']
					cols = parameters['colspan']

				except KeyError:

					rows = None
					cols = None

			self.html_widget = \
				html_widgets.textarea(name=html_name,
					rows=rows,
					cols=cols)

			self.value_expression = parameters['value']

		elif self.type == 'select':

			multiple = parameters.has_key('multiple')

			self.html_widget = \
				html_widgets.select(name=html_name,
					multiple=multiple,
					size=size)

			if parameters.has_key('selected'):

				self.selected_parameter = parameters['selected']

			else:

				self.selected_parameter = ""

			self.options_expression = parameters['options']

		elif self.type == 'boolean':

			self.value_expression = parameters['value']

			try:

				true = parameters['true']
				false = parameters['false']

			except KeyError:

				true = None
				false = None

			self.html_widget = \
				composite_widgets.boolean_widget(name=html_name,
					true=true, false=false)

		elif self.type == 'checkbox_collection':

			self.html_widget = html_widgets.checkbox_collection(name=html_name)

			if parameters.has_key('selected'):

				self.selected_parameter = parameters['selected']

			else:

				self.selected_parameter = ""

			self.options_expression = parameters['options']

		else:

			raise str(parameters)


	def __call__(self, mapping):
		"""Renders the tag."""

		if self.type == 'select':

			options_evaluation = VSEval.Eval(self.options_expression)

			if self.selected_parameter:

				selected_parameter = VSEval.Eval(self.selected_parameter)
				selected_parameter = selected_parameter.eval(mapping)

			else:

				selected_parameter = []

			parameter_options = options_evaluation.eval(mapping)

			self.html_widget = html_widgets.make_select(name=self.html_widget.name,
						size=self.html_widget.size,
						multiple=self.html_widget.multiple,
						options=parameter_options,
						selected_values=selected_parameter)

		elif self.type == 'checkbox_collection':

			options_evaluation = VSEval.Eval(self.options_expression)

			if self.selected_parameter:

				selected_parameter = VSEval.Eval(self.selected_parameter)
				selected_parameter = selected_parameter.eval(mapping)

			else:

				selected_parameter = []

			parameter_options = options_evaluation.eval(mapping)

			self.html_widget = html_widgets.make_checkbox_collection(name=self.html_widget.name,
						options=parameter_options,
						selected_values=selected_parameter)

		elif self.type == 'boolean':

			value = VSEval.Eval(self.value_expression).eval(mapping)
			self.html_widget = composite_widgets.boolean_widget(name=self.html_widget.name,
						selected=value)

		elif self.type == 'text':

			self.html_widget.value = VSEval.Eval(self.value_expression).eval(mapping)

		elif self.type == 'textarea':

			self.html_widget.value = VSEval.Eval(self.value_expression).eval(mapping)

		return self.html_widget(html_quote=self.html_quote)


DT_String.String.commands['widget'] = widget_tag
